@@ -51,10 +51,11 @@ static void setcom(int);
 int decide_tag_by_context(const char *, const char *, int);
 int main(int, char **);
 void completion(const char *, const char *, const char *);
+void completion_idutils(const char *, const char *, const char *);
 void idutils(const char *, const char *);
 void grep(const char *, const char *);
 void pathlist(const char *, const char *);
-void parsefile_using_builtin_parser(char *const *, const char *, const char *, const char *, int);
+void parsefile(char *const *, const char *, const char *, const char *, int);
 int search(const char *, const char *, const char *, const char *, int);
 void tagsearch(const char *, const char *, const char *, const char *, int);
 void encode(char *, int, const char *);
@@ -158,6 +159,10 @@ setcom(int c)
 {
 	if (command == 0)
 		command = c;
+	else if (c == 'c' && command == 'I')
+		command = c;
+	else if (c == 'I' && command == 'c')
+		;
 	else if (command != c)
 		usage();
 }
@@ -497,7 +502,10 @@ main(int argc, char **argv)
 	 * complete function name
 	 */
 	if (cflag) {
-		completion(dbpath, root, av);
+		if (Iflag)
+			completion_idutils(dbpath, root, av);
+		else
+			completion(dbpath, root, av);
 		exit(0);
 	}
 	/*
@@ -583,7 +591,7 @@ main(int argc, char **argv)
 	 * parse source files.
 	 */
 	else if (fflag) {
-		parsefile_using_builtin_parser(argv, cwd, root, dbpath, db);
+		parsefile(argv, cwd, root, dbpath, db);
 	}
 	/*
 	 * tag search.
@@ -663,6 +671,57 @@ completion(const char *dbpath, const char *root, const char *prefix)
 	}
 	gtags_close(gtop);
 }
+/*
+ * completion_idutils: print completion list of specified prefix
+ *
+ *	i)	dbpath	dbpath directory
+ *	i)	root	root directory
+ *	i)	prefix	prefix of primary key
+ */
+void
+completion_idutils(const char *dbpath, const char *root, const char *prefix)
+{
+	FILE *ip;
+	STRBUF *sb = strbuf_open(0);
+	const char *lid = usable("lid");
+	char *line, *p;
+
+	if (prefix && *prefix == 0)	/* In the case global -c '' */
+		prefix = NULL;
+	/*
+	 * make lid command line.
+	 * Invoke lid with the --result=grep option to generate grep format.
+	 */
+	if (!lid)
+		die("lid(idutils) not found.");
+	strbuf_puts(sb, lid);
+	strbuf_puts(sb, " --key=token");
+	if (iflag)
+		strbuf_puts(sb, " --ignore-case");
+	strbuf_putc(sb, ' ');
+	strbuf_putc(sb, '"');
+	strbuf_putc(sb, '^');
+	strbuf_puts(sb, prefix);
+	strbuf_putc(sb, '"');
+	if (debug)
+		fprintf(stderr, "completion_idutils: %s\n", strbuf_value(sb));
+	if (chdir(root) < 0)
+		die("cannot move to '%s' directory.", root);
+	if (!(ip = popen(strbuf_value(sb), "r")))
+		die("cannot execute '%s'.", strbuf_value(sb));
+	while ((line = strbuf_fgets(sb, ip, STRBUF_NOCRLF)) != NULL) {
+		for (p = line; *p && *p != ' '; p++)
+			;
+		if (*p == '\0') {
+			warning("Illegal line: %s", line);
+			continue;
+		}
+		*p = '\0';
+		puts(line);
+	}
+	fclose(ip);
+	strbuf_close(sb);
+}
 /*
  * Output filter
  *
@@ -724,10 +783,11 @@ idutils(const char *pattern, const char *dbpath)
 	FILE *ip;
 	CONVERT *cv;
 	STRBUF *ib = strbuf_open(0);
-	char edit[IDENTLEN];
-	const char *path, *lno, *lid;
+	char encoded_pattern[IDENTLEN];
+	char path[MAXPATHLEN];
+	const char *lid;
 	int linenum, count;
-	char *p, *grep;
+	char *p, *q, *grep;
 
 	lid = usable("lid");
 	if (!lid)
@@ -735,7 +795,7 @@ idutils(const char *pattern, const char *dbpath)
 	/*
 	 * convert spaces into %FF format.
 	 */
-	encode(edit, sizeof(edit), pattern);
+	encode(encoded_pattern, sizeof(encoded_pattern), pattern);
 	/*
 	 * make lid command line.
 	 * Invoke lid with the --result=grep option to generate grep format.
@@ -756,17 +816,19 @@ idutils(const char *pattern, const char *dbpath)
 		die("cannot execute '%s'.", strbuf_value(ib));
 	cv = convert_open(type, format, root, cwd, dbpath, stdout);
 	count = 0;
+	strcpy(path, "./");
 	while ((grep = strbuf_fgets(ib, ip, STRBUF_NOCRLF)) != NULL) {
-		p = grep;
+		q = path + 2;
 		/* extract path name */
-		path = p;
+		if (*grep == '/')
+			die("The path in the output of lid is assumed absolute. '%s'", grep);
+		p = grep;
 		while (*p && *p != ':')
-			p++;
+			*q++ = *p++;
+		*q = '\0'; 
 		if ((xflag || tflag) && !*p)
-			die("invalid lid(idutils) output format. '%s'", grep);
-		*p++ = 0;
-		/* normalize path name */
-		path = makepath(".", path, NULL);
+			die("invalid lid(idutils) output format(1). '%s'", grep);
+		p++;
 		if (lflag) {
 			if (!locatestring(path, localprefix, MATCH_AT_FIRST))
 				continue;
@@ -780,19 +842,18 @@ idutils(const char *pattern, const char *dbpath)
 			/* extract line number */
 			while (*p && isspace(*p))
 				p++;
-			lno = p;
-			while (*p && isdigit(*p))
-				p++;
+			linenum = 0;
+			for (linenum = 0; *p && isdigit(*p); linenum = linenum * 10 + (*p++ - '0'))
+				;
 			if (*p != ':')
-				die("invalid lid(idutils) output format. '%s'", grep);
-			*p++ = 0;
-			linenum = atoi(lno);
+				die("invalid lid(idutils) output format(2). '%s'", grep);
 			if (linenum <= 0)
-				die("invalid lid(idutils) output format. '%s'", grep);
+				die("invalid lid(idutils) output format(3). '%s'", grep);
+			p++;
 			/*
 			 * print out.
 			 */
-			convert_put_using(cv, edit, path, linenum, p, NULL);
+			convert_put_using(cv, encoded_pattern, path, linenum, p, NULL);
 			break;
 		}
 	}
@@ -818,7 +879,7 @@ grep(const char *pattern, const char *dbpath)
 	GFIND *gp;
 	STRBUF *ib = strbuf_open(MAXBUFLEN);
 	const char *path;
-	char edit[IDENTLEN];
+	char encoded_pattern[IDENTLEN];
 	const char *buffer;
 	int linenum, count;
 	int flags = 0;
@@ -828,7 +889,7 @@ grep(const char *pattern, const char *dbpath)
 	/*
 	 * convert spaces into %FF format.
 	 */
-	encode(edit, sizeof(edit), pattern);
+	encode(encoded_pattern, sizeof(encoded_pattern), pattern);
 
 	if (oflag)
 		target = GPATH_BOTH;
@@ -856,7 +917,7 @@ grep(const char *pattern, const char *dbpath)
 					convert_put_path(cv, path);
 					break;
 				} else {
-					convert_put_using(cv, edit, path, linenum, buffer, gp->dbop->lastdat);
+					convert_put_using(cv, encoded_pattern, path, linenum, buffer, gp->dbop->lastdat);
 				}
 			}
 		}
@@ -950,7 +1011,7 @@ pathlist(const char *pattern, const char *dbpath)
 	}
 }
 /*
- * parsefile_using_builtin_parser: parse file to pick up tags.
+ * parsefile: parse file to pick up tags.
  *
  *	i)	argv
  *	i)	cwd	current directory
@@ -1020,7 +1081,7 @@ put_syms(int type, const char *tag, int lno, const char *path, const char *line_
 	data->count++;
 }
 void
-parsefile_using_builtin_parser(char *const *argv, const char *cwd, const char *root, const char *dbpath, int db)
+parsefile(char *const *argv, const char *cwd, const char *root, const char *dbpath, int db)
 {
 	int count = 0;
 	int file_count = 0;
