@@ -59,8 +59,8 @@ static void usage(void);
 static void help(void);
 int main(int, char **);
 int incremental(const char *, const char *);
-void updatetags_using_builtin_parser(const char *, const char *, IDSET *, STRBUF *);
-void createtags_using_builtin_parser(const char *, const char *);
+void updatetags(const char *, const char *, IDSET *, STRBUF *);
+void createtags(const char *, const char *);
 int printconf(const char *);
 
 int cflag;					/* compact format */
@@ -82,6 +82,8 @@ const char *dump_target;
 char *single_update;
 int statistics = STATISTICS_STYLE_NONE;
 
+#define GTAGSFILES "gtags.files"
+
 /*
  * Path filter
  */
@@ -335,8 +337,11 @@ main(int argc, char **argv)
 	}
 
 	/*
+	 * If 'gtags.files' exists, use it as a file list.
 	 * If the file_list other than "-" is given, it must be readable file.
 	 */
+	if (file_list == NULL && test("f", GTAGSFILES))
+		file_list = GTAGSFILES;
 	if (file_list && strcmp(file_list, "-")) {
 		if (test("d", file_list))
 			die("'%s' is a directory.", file_list);
@@ -420,6 +425,8 @@ main(int argc, char **argv)
 	if (vflag && gtags_parser)
 		fprintf(stderr, " Using plug-in parser.\n");
 	parser_init(langmap, gtags_parser);
+	if (vflag && file_list)
+		fprintf(stderr, " Using '%s' as a file list.\n", file_list);
 	/*
 	 * incremental update.
 	 */
@@ -444,7 +451,7 @@ main(int argc, char **argv)
 	/*
 	 * create GTAGS and GRTAGS
 	 */
-	createtags_using_builtin_parser(dbpath, cwd);
+	createtags(dbpath, cwd);
 	/*
 	 * create idutils index.
 	 */
@@ -623,7 +630,7 @@ normal_update:
 	 * execute updating.
 	 */
 	if (!idset_empty(deleteset) || strbuf_getlen(addlist) > 0) {
-		updatetags_using_builtin_parser(dbpath, root, deleteset, addlist);
+		updatetags(dbpath, root, deleteset, addlist);
 		updated = 1;
 	}
 	if (strbuf_getlen(deletelist) + strbuf_getlen(addlist_other) > 0) {
@@ -701,7 +708,7 @@ put_syms(int type, const char *tag, int lno, const char *path, const char *line_
 	gtags_put_using(gtop, tag, lno, data->fid, line_image);
 }
 /*
- * updatetags_using_builtin_parser: update tag file.
+ * updatetags: update tag file.
  *
  *	i)	dbpath		directory in which tag file exist
  *	i)	root		root directory of source tree
@@ -709,7 +716,7 @@ put_syms(int type, const char *tag, int lno, const char *path, const char *line_
  *	i)	addlist		\0 separated list of added or modified files
  */
 void
-updatetags_using_builtin_parser(const char *dbpath, const char *root, IDSET *deleteset, STRBUF *addlist)
+updatetags(const char *dbpath, const char *root, IDSET *deleteset, STRBUF *addlist)
 {
 	struct put_func_data data;
 	int seqno, flags;
@@ -755,12 +762,15 @@ updatetags_using_builtin_parser(const char *dbpath, const char *root, IDSET *del
 	/*
 	 * Set flags.
 	 */
-	flags = 0;
+	data.gtop[GTAGS]->flags = 0;
 	if (extractmethod)
-		flags |= GTAGS_EXTRACTMETHOD;
+		data.gtop[GTAGS]->flags |= GTAGS_EXTRACTMETHOD;
+	data.gtop[GRTAGS]->flags = data.gtop[GTAGS]->flags;
+	flags = 0;
 	if (debug)
-		flags |= GTAGS_DEBUG;
-	data.gtop[GTAGS]->flags = data.gtop[GRTAGS]->flags = flags;
+		flags |= PARSER_DEBUG;
+	if (wflag)
+		flags |= PARSER_WARNING;
 	/*
 	 * Add tags to GTAGS and GRTAGS.
 	 */
@@ -774,7 +784,9 @@ updatetags_using_builtin_parser(const char *dbpath, const char *root, IDSET *del
 			die("GPATH is corrupted.('%s' not found)", path);
 		if (vflag)
 			fprintf(stderr, " [%d/%d] extracting tags of %s\n", ++seqno, total, path + 2);
-		parse_file(path, wflag ? PARSER_WARNING : 0, put_syms, &data);
+		if (debug)
+			fprintf(stderr, "[%s]\n", path + 2);
+		parse_file(path, flags, put_syms, &data);
 		gtags_flush(data.gtop[GTAGS], data.fid);
 		if (data.gtop[GRTAGS] != NULL)
 			gtags_flush(data.gtop[GRTAGS], data.fid);
@@ -785,18 +797,18 @@ updatetags_using_builtin_parser(const char *dbpath, const char *root, IDSET *del
 		gtags_close(data.gtop[GRTAGS]);
 }
 /*
- * createtags_using_builtin_parser: create tags file
+ * createtags: create tags file
  *
  *	i)	dbpath	dbpath directory
  *	i)	root	root directory of source tree
  */
 void
-createtags_using_builtin_parser(const char *dbpath, const char *root)
+createtags(const char *dbpath, const char *root)
 {
 	STATISTICS_TIME *tim;
 	STRBUF *sb = strbuf_open(0);
 	struct put_func_data data;
-	int openflags, seqno;
+	int openflags, flags, seqno;
 	const char *path;
 
 	tim = statistics_time_start("Time of creating %s and %s.", dbname(GTAGS), dbname(GRTAGS));
@@ -807,10 +819,13 @@ createtags_using_builtin_parser(const char *dbpath, const char *root)
 	data.gtop[GTAGS]->flags = 0;
 	if (extractmethod)
 		data.gtop[GTAGS]->flags |= GTAGS_EXTRACTMETHOD;
-	if (debug)
-		data.gtop[GTAGS]->flags |= GTAGS_DEBUG;
 	data.gtop[GRTAGS] = gtags_open(dbpath, root, GRTAGS, GTAGS_CREATE, openflags);
 	data.gtop[GRTAGS]->flags = data.gtop[GTAGS]->flags;
+	flags = 0;
+	if (debug)
+		flags |= PARSER_DEBUG;
+	if (wflag)
+		flags |= PARSER_WARNING;
 	/*
 	 * Add tags to GTAGS and GRTAGS.
 	 */
@@ -833,7 +848,9 @@ createtags_using_builtin_parser(const char *dbpath, const char *root)
 		seqno++;
 		if (vflag)
 			fprintf(stderr, " [%d] extracting tags of %s\n", seqno, path + 2);
-		parse_file(path, wflag ? PARSER_WARNING : 0, put_syms, &data);
+		if (debug)
+			fprintf(stderr, "[%s]\n", path + 2);
+		parse_file(path, flags, put_syms, &data);
 		gtags_flush(data.gtop[GTAGS], data.fid);
 		gtags_flush(data.gtop[GRTAGS], data.fid);
 	}
