@@ -37,6 +37,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/param.h>
+#include <errno.h>
 
 #include "checkalloc.h"
 #include "getopt.h"
@@ -117,6 +118,11 @@ int caution;				/* --caution option		*/
 int dynamic;				/* --dynamic(-D) option		*/
 int symbol;				/* --symbol(-s) option          */
 int suggest;				/* --suggest option		*/
+int suggest2;				/* --suggest2 option		*/
+int auto_completion;			/* --auto-completion		*/
+int tree_view;				/* --tree-view			*/
+const char *tree_view_type;		/* -- type-view=[type]		*/
+char *auto_completion_limit = "0";	/* --auto-completion=limit	*/
 int statistics = STATISTICS_STYLE_NONE;	/* --statistics option		*/
 
 int no_order_list;			/* 1: doesn't use order list	*/
@@ -137,7 +143,7 @@ const char *title;
 const char *xhtml_version = "1.0";
 const char *insert_header;		/* --insert-header=<file>	*/
 const char *insert_footer;		/* --insert-footer=<file>	*/
-
+const char *jscode;			/* javascript code		*/
 /*
  * Constant values.
  */
@@ -222,6 +228,7 @@ int tabs = 8;				/* tab skip			*/
 int flist_fields = 5;			/* fields number of file list	*/
 int full_path = 0;			/* file index format		*/
 int map_file = 1;			/* 1: create MAP file		*/
+int overwrite_key = 0;			/* 0: over write site key	*/
 const char *icon_suffix = "png";	/* icon suffix (jpg, png etc)	*/
 const char *icon_spec = "border='0' align='top'";/* parameter in IMG tag*/
 const char *prolog_script = NULL;	/* include script at first	*/
@@ -236,9 +243,9 @@ const char *gzipped_suffix = "ghtml";	/* suffix of gzipped html file	*/
 const char *normal_suffix = "html";	/* suffix of normal html file	*/
 const char *HTML;
 const char *action = "cgi-bin/global.cgi";/* default action		*/
-const char *saction;			/* safe action			*/
-const char *id = NULL;			/* id (default non)		*/
-int cgi = 1;				/* 1: make cgi-bin/		*/
+const char *completion_action = "cgi-bin/completion.cgi";
+const char *id = "";			/* id (default non)		*/
+int nocgi = 0;				/* 1: don't make cgi-bin/	*/
 int definition_header=NO_HEADER;	/* (NO|BEFORE|RIGHT|AFTER)_HEADER */
 const char *htags_options = NULL;
 const char *include_file_suffixes = "h,hxx,hpp,H,inc.php";
@@ -279,11 +286,13 @@ static struct option const long_options[] = {
         {"disable-idutils", no_argument, &enable_idutils, 0},
         {"full-path", no_argument, &full_path, 1},
         {"html", no_argument, &enable_xhtml, 0},
-        {"nocgi", no_argument, &cgi, 0},
+        {"nocgi", no_argument, &nocgi, 1},
         {"no-map-file", no_argument, &map_file, 0},
+        {"overwrite-key", no_argument, &overwrite_key, 1},
         {"show-position", no_argument, &show_position, 1},
         {"statistics", no_argument, &statistics, STATISTICS_STYLE_TABLE},
         {"suggest", no_argument, &suggest, 1},
+        {"suggest2", no_argument, &suggest2, 1},
         {"table-list", no_argument, &table_list, 1},
         {"version", no_argument, &show_version, 1},
         {"help", no_argument, &show_help, 1},
@@ -301,7 +310,10 @@ static struct option const long_options[] = {
 #define OPT_ITEM_ORDER		137
 #define OPT_TABS		138
 #define OPT_CFLOW		139
+#define OPT_AUTO_COMPLETION	140
+#define OPT_TREE_VIEW		141
         {"action", required_argument, NULL, OPT_ACTION},
+        {"auto-completion", optional_argument, NULL, OPT_AUTO_COMPLETION},
         {"cflow", required_argument, NULL, OPT_CFLOW},
         {"cvsweb", required_argument, NULL, OPT_CVSWEB},
         {"cvsweb-cvsroot", required_argument, NULL, OPT_CVSWEB_CVSROOT},
@@ -313,6 +325,7 @@ static struct option const long_options[] = {
         {"insert-header", required_argument, NULL, OPT_INSERT_HEADER},
         {"item-order", required_argument, NULL, OPT_ITEM_ORDER},
 	{"tabs", required_argument, NULL, OPT_TABS},
+        {"tree-view",  optional_argument, NULL, OPT_TREE_VIEW},
         { 0 }
 };
 
@@ -375,9 +388,10 @@ signal_setup(void)
 static void
 make_directory_in_distpath(const char *name)
 {
-	const char *path = makepath(distpath, name, NULL);
+	char path[MAXPATHLEN];
 	FILE *op;
 
+	strlimcpy(path, makepath(distpath, name, NULL), sizeof(path));
 	if (!test("d", path))
 		if (mkdir(path, 0775))
 			die("cannot make directory '%s'.", path);
@@ -385,54 +399,75 @@ make_directory_in_distpath(const char *name)
 	 * Not to publish the directory list.
 	 */
 	op = fopen(makepath(path, "index.html", NULL), "w");
+	if (op == NULL)
+		die("cannot make file '%s'.", makepath(path, "index.html", NULL));
 	fputs(html_begin, op);
 	fputs(html_end, op);
 	fputc('\n', op);
 	fclose(op);
 }
 /*
- * generate_file: generate file with replacing macro.
- *
- *	i)	dist	directory where the file should be created
- *	i)	file	file name
- *	i)	place	TOPDIR, SUBDIR, CGIDIR
+ * make file in the dist directory.
  */
 static void
-generate_file(const char *dist, const char *file, int place)
+make_file_in_distpath(const char *name, const char *data)
 {
+	FILE *op;
+	const char *path = makepath(distpath, name, NULL);
+
+	op = fopen(path, "w");
+	if (op) {
+		if (data) {
+			fputs(data, op);
+			fputc('\n', op);
+		}
+		fclose(op);
+	} else {
+		die("cannot make file '%s'.", path); 
+	}
+}
+void
+load_with_replace(const char *file, STRBUF *result, int place)
+{
+	STRBUF *sb = strbuf_open(0);
+	FILE *ip;
 	regex_t preg;
 	regmatch_t pmatch[2];
-	STRBUF *sb = strbuf_open(0);
-	FILE *ip, *op;
 	char *_;
 	int i;
+
         struct map {
-                const char *name;
-                const char *value;
+		const char *name;
+		const char *value;
         } tab[] = {
 		/* dynamic initialization */
-                {"@page_begin@", NULL},
-                {"@page_end@", NULL},
+		{"@page_begin@", NULL},
+		{"@page_end@", NULL},
 
 		/* static initialization */
-                {"@body_begin@", body_begin},
-                {"@body_end@", body_end},
-                {"@title_begin@", title_begin},
-                {"@title_end@", title_end},
-                {"@error_begin@", error_begin},
-                {"@error_end@", error_end},
-                {"@message_begin@", message_begin},
-                {"@message_end@", message_end},
-                {"@verbatim_begin@", verbatim_begin},
-                {"@verbatim_end@", verbatim_end},
-                {"@normal_suffix@", normal_suffix},
-                {"@hr@", hr},
-                {"@br@", br},
-                {"@HTML@", HTML},
-                {"@action@", action},
-                {"@null_device@", null_device},
-                {"@globalpath@", global_path},
-                {"@gtagspath@", gtags_path},
+		{"@body_begin@", body_begin},
+		{"@body_end@", body_end},
+		{"@title_begin@", title_begin},
+		{"@title_end@", title_end},
+		{"@error_begin@", error_begin},
+		{"@error_end@", error_end},
+		{"@message_begin@", message_begin},
+		{"@message_end@", message_end},
+		{"@verbatim_begin@", verbatim_begin},
+		{"@verbatim_end@", verbatim_end},
+		{"@normal_suffix@", normal_suffix},
+		{"@hr@", hr},
+		{"@br@", br},
+		{"@HTML@", HTML},
+		{"@DATADIR@", datadir},
+		{"@action@", action},
+		{"@completion_action@", completion_action},
+		{"@limit@", auto_completion_limit},
+		{"@script_alias@", script_alias},
+		{"@id@", id},
+		{"@null_device@", null_device},
+		{"@globalpath@", global_path},
+		{"@gtagspath@", gtags_path},
         };
 	int tabsize = sizeof(tab) / sizeof(struct map);
 
@@ -465,9 +500,6 @@ generate_file(const char *dist, const char *file, int place)
 #endif
 			die("skeleton file '%s' not found.", strbuf_value(sb));
 	}
-	op = fopen(makepath(dist, file, NULL), "w");
-	if (!op)
-		die("cannot create file '%s'.", file);
 	strbuf_reset(sb);
 	/*
 	 * Read template file and evaluate macros.
@@ -482,7 +514,7 @@ generate_file(const char *dist, const char *file, int place)
 
 			/* print before macro */
 			for (i = 0; i < pmatch[0].rm_so; i++)
-				fputc(p[i], op);
+				strbuf_putc(result, p[i]);
 			for (i = 0; i < tabsize; i++)
 				if (!strncmp(start, tab[i].name, length))
 					break;
@@ -496,22 +528,41 @@ generate_file(const char *dist, const char *file, int place)
 				 */
 				for (q = tab[i].value; *q; q++) {
 					if (*q == '"')
-						fputc('\\', op);
+						strbuf_putc(result, '\\');
 					else if (*q == '\n')
-						fputc('\\', op);
-					fputc(*q, op);
+						strbuf_putc(result, '\\');
+					strbuf_putc(result, *q);
 				}
 			}
 		}
-		fputs_nl(p, op);
+		strbuf_puts_nl(result, p);
 	}
-	fclose(op);
 	fclose(ip);
 	strbuf_close(sb);
 	regfree(&preg);
-	html_count++;
 }
+/*
+ * generate_file: generate file with replacing macro.
+ *
+ *	i)	dist	directory where the file should be created
+ *	i)	file	file name
+ *	i)	place	TOPDIR, SUBDIR, CGIDIR
+ */
+static void
+generate_file(const char *dist, const char *file, int place)
+{
+	FILE *op;
+	STRBUF *result = strbuf_open(0);
 
+	op = fopen(makepath(dist, file, NULL), "w");
+	if (!op)
+		die("cannot create file '%s'.", file);
+	load_with_replace(file, result, place);
+	fputs(strbuf_value(result), op);
+	fclose(op);
+	html_count++;
+	strbuf_close(result);
+}
 /*
  * makeprogram: make CGI program
  */
@@ -526,10 +577,7 @@ makeprogram(const char *cgidir, const char *file)
 static void
 makebless(const char *file)
 {
-	const char *save = action;
-	action = saction;
 	generate_file(distpath, file, SUBDIR);
-	action = save;
 }
 /*
  * makeghtml: make ghtml.cgi file.
@@ -632,13 +680,11 @@ makehelp(const char *file)
 /*
  * makesearchpart: make search part
  *
- *	i)	$action	action url
- *	i)	$id	hidden variable
  *	i)	$target	target
  *	r)		html
  */
 static char *
-makesearchpart(const char *action, const char *id, const char *target)
+makesearchpart(const char *target)
 {
 	STATIC_STRBUF(sb);
 
@@ -656,8 +702,6 @@ makesearchpart(const char *action, const char *id, const char *target)
 	}
 	strbuf_puts_nl(sb, gen_form_begin(target));
 	strbuf_puts_nl(sb, gen_input("pattern", NULL, NULL));
-	if (id == NULL)
-		id = "";
 	strbuf_puts_nl(sb, gen_input("id", id, "hidden"));
 	strbuf_puts_nl(sb, gen_input(NULL, "Search", "submit"));
 	strbuf_puts(sb, gen_input(NULL, "Reset", "reset"));
@@ -669,7 +713,7 @@ makesearchpart(const char *action, const char *id, const char *target)
 	strbuf_puts(sb, gen_input_radio("type", "symbol", 0, "Retrieve the place of the specified symbol is used."));
 	strbuf_puts_nl(sb, target ? "Sym" : "Other symbol");
 	strbuf_puts(sb, gen_input_radio("type", "path", 0, "Look for path name which matches to the specified pattern."));
-	strbuf_puts(sb, target ? "Path" : "Path name");
+	strbuf_puts_nl(sb, target ? "Path" : "Path name");
 	if (enable_grep) {
 		strbuf_puts(sb, gen_input_radio("type", "grep", 0, "Retrieve lines which matches to the specified pattern."));
 		strbuf_puts_nl(sb, target ? "Grep" : "Grep pattern");
@@ -679,10 +723,10 @@ makesearchpart(const char *action, const char *id, const char *target)
 		strbuf_puts_nl(sb, target ? "Id" : "Id pattern");
 	}
 	strbuf_puts_nl(sb, br);
-	strbuf_puts(sb, gen_input_checkbox("icase", "1", "Ignore case distinctions in the pattern."));
+	strbuf_puts(sb, gen_input_checkbox("icase", NULL, "Ignore case distinctions in the pattern."));
 	strbuf_puts_nl(sb, target ? "Icase" : "Ignore case");
 	if (other_files) {
-		strbuf_puts(sb, gen_input_checkbox("other", "1", "Files other than the source code are also retrieved."));
+		strbuf_puts(sb, gen_input_checkbox("other", NULL, "Files other than the source code are also retrieved."));
 		strbuf_puts_nl(sb, target ? "Other" : "Other files");
 	}
 	if (other_files && !target) {
@@ -735,7 +779,7 @@ makeindex(const char *file, const char *title, const char *index)
 		fputs_nl(gen_frameset_end(), op);
 		fputs_nl(gen_page_end(), op);
 	} else {
-		fputs_nl(gen_page_begin(title, TOPDIR), op);
+		fputs_nl(gen_page_index_begin(title, jscode), op);
 		fputs_nl(body_begin, op);
 		if (insert_header)
 			fputs(gen_insert_header(TOPDIR), op);
@@ -762,7 +806,7 @@ makemainindex(const char *file, const char *index)
 	op = fopen(makepath(distpath, file, NULL), "w");
 	if (!op)
 		die("cannot make file '%s'.", file);
-	fputs_nl(gen_page_begin(title, TOPDIR), op);
+	fputs_nl(gen_page_index_begin(title, jscode), op);
 	fputs_nl(body_begin, op);
 	if (insert_header)
 		fputs(gen_insert_header(TOPDIR), op);
@@ -787,9 +831,9 @@ makesearchindex(const char *file)
 	op = fopen(makepath(distpath, file, NULL), "w");
 	if (!op)
 		die("cannot create file '%s'.", file);
-	fputs_nl(gen_page_begin("SEARCH", TOPDIR), op);
+	fputs_nl(gen_page_index_begin("SEARCH", jscode), op);
 	fputs_nl(body_begin, op);
-	fputs(makesearchpart(action, id, "mains"), op);
+	fputs(makesearchpart("mains"), op);
 	fputs_nl(body_end, op);
 	fputs_nl(gen_page_end(), op);
 	fclose(op);
@@ -893,6 +937,14 @@ makehtml(int total)
 	}
 	gfind_close(gp);
 }
+/*
+ * Load file.
+ */
+void
+loadfile(const char *file, STRBUF *result)
+{
+	load_with_replace(file, result, 0);
+}
 /*
  * copy file.
  */
@@ -981,7 +1033,7 @@ makecommonpart(const char *title, const char *defines, const char *files)
 			break;
 		case 's':
 			if (fflag) {
-				strbuf_puts(sb, makesearchpart(action, id, NULL));
+				strbuf_puts(sb, makesearchpart(NULL));
 				strbuf_puts_nl(sb, hr);
 			}
 			break;
@@ -1040,12 +1092,17 @@ makecommonpart(const char *title, const char *defines, const char *files)
 				strbuf_puts(sb, header_begin);
 				strbuf_puts(sb, title_file_index);
 				strbuf_puts_nl(sb, header_end);
-				if (table_flist)
+				if (tree_view) {
+					strbuf_puts_nl(sb, tree_control);
+					strbuf_puts_nl(sb, tree_begin);
+				} else if (table_flist)
 					strbuf_puts_nl(sb, flist_begin);
 				else if (!no_order_list)
 					strbuf_puts_nl(sb, list_begin);
 				strbuf_puts(sb, files);
-				if (table_flist)
+				if (tree_view)
+					strbuf_puts_nl(sb, tree_end);
+				else if (table_flist)
 					strbuf_puts_nl(sb, flist_end);
 				else if (!no_order_list)
 					strbuf_puts_nl(sb, list_end);
@@ -1169,14 +1226,6 @@ configuration(int argc, char *const *argv)
 		normal_suffix = check_strdup(strbuf_value(sb));
 	if (getconfb("no_order_list"))
 		no_order_list = 1;
-/*
-	strbuf_reset(sb);
-	if (getconfs("icon_spec", sb))
-		icon_spec = check_strdup(strbuf_value(sb));
-	strbuf_reset(sb);
-	if (getconfs("icon_suffix", sb))
-		icon_suffix = check_strdup(strbuf_value(sb));
-*/
 	strbuf_reset(sb);
 	if (getconfs("prolog_script", sb))
 		prolog_script = check_strdup(strbuf_value(sb));
@@ -1475,6 +1524,15 @@ main(int argc, char **argv)
 		case OPT_ACTION:
 			action_value = optarg;
 			break;
+		case OPT_AUTO_COMPLETION:
+			auto_completion = 1;
+			if (optarg) {
+				if (atoi(optarg) > 0)
+					auto_completion_limit = optarg;
+				else
+					die("The option value of --auto-completion must be numeric.");
+			}
+			break;
 		case OPT_CFLOW:
 			cflow_file = optarg;
 			break;
@@ -1505,6 +1563,11 @@ main(int argc, char **argv)
 			else
 				die("--tabs option requires numeric value.");
                         break;
+		case OPT_TREE_VIEW:
+			tree_view = 1;
+			if (optarg)
+				tree_view_type = optarg;
+			break;
                 case 'a':
                         aflag++;
                         break;
@@ -1562,7 +1625,7 @@ main(int argc, char **argv)
                         break;
                 case 'S':
 			Sflag++;
-			cgidir = optarg;
+			id = optarg;
                         break;
                 case 'T':
 			table_flist = 1;
@@ -1602,18 +1665,17 @@ main(int argc, char **argv)
 	}
 	/*
 	 * Leaving everything to htags.
-	 * Htags selects the most popular options for you.
-	 * Htags likes busy screen but dislikes frameed screen.
-	 * You may want to invoke htags with "htags --leave --frame".
-	 * Htags also make tag files if not found.
+	 * Htags selects popular options for you.
 	 */
+	if (suggest2)
+		suggest = 1;
 	if (suggest) {
 		int gtags_not_found = 0;
 
-		aflag = fflag = Iflag = nflag = vflag = 1;
+		aflag = Iflag = nflag = vflag = 1;
 		setverbose();
 		definition_header = AFTER_HEADER;
-		other_files = symbol = enable_xhtml = show_position = table_flist = 1;
+		other_files = symbol = show_position = table_flist = 1;
 		if (arg_dbpath[0]) {
 			if (!test("f", makepath(arg_dbpath, dbname(GTAGS), NULL)))
 				gtags_not_found = 1;
@@ -1624,12 +1686,19 @@ main(int argc, char **argv)
 		if (gtags_not_found)
 			gflag = 1;
 	}
+	if (suggest2) {
+		Fflag = 1;				/* uses frame */
+		cflag = fflag = dynamic = 1;		/* needs a HTTP server */
+		auto_completion = tree_view = 1;	/* needs javascript */
+	}
 	if (cflow_file && !test("fr", cflow_file))
 		die("cflow file not found. '%s'", cflow_file);
 	if (insert_header && !test("fr", insert_header))
 		die("page header file '%s' not found.", insert_header);
 	if (insert_footer && !test("fr", insert_footer))
 		die("page footer file '%s' not found.", insert_footer);
+	if (!fflag)
+		auto_completion = 0;
         argc -= optind;
         argv += optind;
         if (!av)
@@ -1691,8 +1760,6 @@ main(int argc, char **argv)
 		char *p = strrchr(cwdpath, sep);
 		title = p ? p + 1 : cwdpath;
 	}
-	if (dynamic && Sflag)
-		die("Current implementation doesn't allow both -D(--dynamic) and the -S(--secure-cgi).");
 	if (cvsweb_url && test("d", "CVS"))
 		use_cvs_module = 1;
 	/*
@@ -1713,14 +1780,41 @@ main(int argc, char **argv)
 	} else {
 		snprintf(distpath, sizeof(distpath), "%s/HTML", cwdpath);
 	}
-	{
-		static char buf[MAXBUFLEN];
-		snprintf(buf, sizeof(buf), "%s/global.cgi", script_alias);
-		saction = buf;
-	}
 	if (Sflag) {
+		static char saction[MAXBUFLEN];
+		static char completion_saction[MAXBUFLEN];
+		char path[MAXBUFLEN];
+		char *name = "sitekeys";
+		int fd;
+
+		snprintf(saction, sizeof(saction), "%s/global.cgi", script_alias);
 		action = saction;
-		id = distpath;
+		snprintf(completion_saction, sizeof(completion_saction), "%s/completion.cgi", script_alias);
+		completion_action = completion_saction;
+		snprintf(path, sizeof(path), "%s/gtags", datadir);
+		if (!test("d", makepath(path, name, NULL))) {
+			setverbose();
+			message("htags: cannot make sitekey file.");
+			message("\n[Information]\n");
+			message("Htags was invoked with the -S option. It is required a special site key directory.");
+			message("Please make it by the following command line:");
+			message(" $ mkdir %s/%s", path, name);
+			message(" $ chmod 773 %s/%s", path, name);
+			message("");
+			message("Thank you for your cooperation.");
+			exit(0);
+		}
+		snprintf(path, sizeof(path), "%s/gtags/%s/%s", datadir, name, id);
+		if (test("f", path) && overwrite_key == 0)
+			die("key '%s' is not unique. please change key or use --overwrite-key option.", id);
+		fd = creat(path, 644);
+		if (fd < 0)
+			die("cannot create file '%s'.", path);
+		write(fd, distpath, strlen(distpath));
+		write(fd, "\n", 1);
+		if (fchmod(fd, 0644) < 0)
+			die("cannot chmod file '%s'(errono = %d).", path, errno);
+		close(fd);
 	}
 	/* --action, --id overwrite Sflag's value. */
 	if (action_value) {
@@ -1783,16 +1877,12 @@ main(int argc, char **argv)
 	 * check directories
 	 */
 	if (fflag || cflag || dynamic) {
-		if (cgidir && !test("d", cgidir))
-			die("'%s' not found.", cgidir);
-		if (!Sflag) {
-			static char buf[MAXPATHLEN];
-			snprintf(buf, sizeof(buf), "%s/cgi-bin", distpath);
-			cgidir = buf;
-		}
+		static char buf[MAXPATHLEN];
+		snprintf(buf, sizeof(buf), "%s/cgi-bin", distpath);
+		cgidir = buf;
 	} else {
 		Sflag = 0;
-		cgidir = 0;
+		cgidir = NULL;
 	}
 	/*------------------------------------------------------------------
 	 * MAKE FILES
@@ -1834,6 +1924,16 @@ main(int argc, char **argv)
 	if (!w32) {
 		/* UNDER CONSTRUCTION */
 	}
+	if (auto_completion || tree_view) {
+		STATIC_STRBUF(sb);
+		strbuf_clear(sb);
+		strbuf_puts_nl(sb, "<script type='text/javascript' src='js/jquery.js'></script>");
+		if (auto_completion)
+			loadfile("jscode_suggest", sb);
+		if (tree_view)
+			loadfile("jscode_treeview", sb);
+		jscode = strbuf_value(sb);
+	}
 	/*
 	 * (0) make directories
 	 */
@@ -1846,30 +1946,42 @@ main(int argc, char **argv)
 	make_directory_in_distpath(SRCS);
 	make_directory_in_distpath(INCS);
 	make_directory_in_distpath(INCREFS);
+	make_file_in_distpath("sitekey", id);
 	if (!dynamic) {
 		make_directory_in_distpath(DEFS);
 		make_directory_in_distpath(REFS);
 		if (symbol)
 			make_directory_in_distpath(SYMS);
 	}
-	if (cgi && (fflag || cflag || dynamic))
+	if (!nocgi && (fflag || cflag || dynamic))
 		make_directory_in_distpath("cgi-bin");
 	if (Iflag)
 		make_directory_in_distpath("icons");
+	if (auto_completion || tree_view)
+		 make_directory_in_distpath("js");
 	/*
 	 * (1) make CGI program
 	 */
-	if (cgi && (fflag || cflag || dynamic)) {
+	if (!nocgi && (fflag || cflag || dynamic)) {
 		message("[%s] (1) making CGI program ...", now());
 		if (cgidir) {
+			/*
+			 * If the Sflag is specified, CGI script is invalidated.
+			 */
+			int perm = Sflag ? 0644 : 0755;
 			if (fflag || dynamic) {
 				makeprogram(cgidir, "global.cgi");
-				if (chmod(makepath(cgidir, "global.cgi", NULL), 0755) < 0)
+				if (chmod(makepath(cgidir, "global.cgi", NULL), perm) < 0)
+					die("cannot chmod CGI program.");
+			}
+			if (auto_completion) {
+				makeprogram(cgidir, "completion.cgi");
+				if (chmod(makepath(cgidir, "completion.cgi", NULL), perm) < 0)
 					die("cannot chmod CGI program.");
 			}
 			if (cflag) {
 				makeghtml(cgidir, "ghtml.cgi");
-				if (chmod(makepath(cgidir, "ghtml.cgi", NULL), 0755) < 0)
+				if (chmod(makepath(cgidir, "ghtml.cgi", NULL), perm) < 0)
 					die("cannot chmod unzip script.");
 			}
 			makehtaccess(".htaccess");
@@ -1889,12 +2001,22 @@ main(int argc, char **argv)
 	/*
 	 * Save the suffix of compress format for the safe CGI script.
 	 */
-	{
-		FILE *op = fopen(makepath(distpath, "compress", NULL), "w");
-		if (cflag) {
-			fputs(HTML, op);
-			fputc('\n', op);
-		}
+	if (cflag) {
+		const char *path = makepath(distpath, "compress", NULL);
+		FILE *op = fopen(path, "w");
+		if (op == NULL)
+			die("cannot make file '%s'.", path);
+		fputs(HTML, op);
+		fputc('\n', op);
+		fclose(op);
+	}
+	if (av) {
+		const char *path = makepath(distpath, "GTAGSROOT", NULL);
+		FILE *op = fopen(path, "w");
+		if (op == NULL)
+			die("cannot make file '%s'.", path);
+		fputs(cwdpath, op);
+		fputc('\n', op);
 		fclose(op);
 	}
 	/*
@@ -2005,36 +2127,34 @@ main(int argc, char **argv)
 	 * (11) style sheet file (style.css)
 	 */
 	if (enable_xhtml) {
-		char src[MAXPATHLEN], dst[MAXPATHLEN];
-		snprintf(src, sizeof(src), "%s/gtags/style.css", datadir);
-		snprintf(dst, sizeof(dst), "%s/style.css", distpath);
-		copyfile(src, dst);
+		char com[MAXPATHLEN*2+32];
+		snprintf(com, sizeof(com), "cp %s/gtags/style.css %s", datadir, distpath);
+		system(com);
+	}
+	if (auto_completion || tree_view) {
+		char com[MAXPATHLEN*2+32];
+		snprintf(com, sizeof(com), "cp -r %s/gtags/jquery/* %s/js", datadir, distpath);
+		system(com);
 	}
 	message("[%s] Done.", now());
-	if (vflag && cgi && (cflag || fflag || dynamic)) {
+	if (vflag && !nocgi && (cflag || fflag || dynamic || tree_view)) {
 		message("\n[Information]\n");
-		if (fflag || cflag || dynamic) {
-			message(" o Htags was invoked with the -f, -c or -D option. You should start HTTP");
-			message("   server so that cgi-bin/*.cgi is executed as a CGI script.");
+		if (cflag || fflag || dynamic || tree_view) {
+			message(" o Htags was invoked with the -f, -c, -D or --tree-view option. You should");
+			message("   start a http server so that cgi-bin/*.cgi is executed as a CGI script.");
 		}
 		if (cflag) {
-			message(" o Htags was invoked with the -c option. You should start HTTP server to");
+			message(" o Htags was invoked with the -c option. You should start a http server to");
 			message("   decompress *.%s files.", gzipped_suffix);
 		}
- 		message(" If you are using Apache, 'HTML/.htaccess' might be helpful for you.\n");
+ 		message("\n If you are using Apache, 'HTML/.htaccess' might be helpful for you.\n");
 		message(" Good luck!\n");
 	}
 	if (Iflag) {
-		char src[MAXPATHLEN], dst[MAXPATHLEN];
-		int i, count = sizeof(icon_files) / sizeof(char *);
+		char com[MAXPATHLEN*2+32];
 
-		for (i = 0; i < count; i++) {
-			snprintf(src, sizeof(src), "%s/gtags/icons/%s.%s", datadir, icon_files[i], icon_suffix);
-			snprintf(dst, sizeof(dst), "%s/icons/%s.%s", distpath, icon_files[i], icon_suffix);
-			if (!test("f", src))
-				die("Icon file '%s' not found.", src);
-			copyfile(src, dst);
-		}
+		snprintf(com, sizeof(com), "cp -r %s/gtags/icons %s", datadir, distpath);
+		system(com);
 	}
 	gpath_close();
 	/*
