@@ -66,6 +66,7 @@ extern char global_path[MAXFILLEN];
 extern int gtags_exist[GTAGLIM];
 extern const char *null_device;
 extern const char *tmpdir;
+extern const char *tree_view_type;
 
 extern int aflag;
 extern int cflag;
@@ -83,6 +84,8 @@ extern int debug;
 extern int show_help;
 extern int show_version;
 extern int caution;
+extern int auto_completion;
+extern int tree_view;
 extern int dynamic;
 extern int symbol;
 extern int statistics;
@@ -105,6 +108,7 @@ extern const char *title;
 extern const char *xhtml_version;
 extern const char *insert_header;
 extern const char *insert_footer;
+extern const char *jscode;
 
 extern const char *title_define_index;
 extern const char *title_file_index;
@@ -138,7 +142,6 @@ extern const char *gzipped_suffix;
 extern const char *normal_suffix;
 extern const char *HTML;
 extern const char *action;
-extern const char *saction;
 extern const char *id;
 extern int cgi;
 extern int definition_header;
