@@ -655,6 +655,11 @@ find_read_filelist(void)
 			/* skip empty line.  */
 			continue;
 		}
+		/*
+		 * Lines which start with ". " are considered to be comments.
+		 */
+		if (*path == '.' && *(path + 1) == ' ')
+			continue;
 		/*
 		 * Skip the following:
 		 * o directory
