@@ -73,13 +73,14 @@ typedef struct {
 
 typedef struct {
 	DBOP *dbop;			/* descripter of DBOP */
+	DBOP *gtags;			/* descripter of GTAGS */
 	int format_version;		/* format version */
 	int format;			/* GTAGS_COMPACT, GTAGS_COMPRESS */
 	int mode;			/* mode */
 	int db;				/* 0:GTAGS, 1:GRTAGS, 2:GSYMS */
 	int openflags;			/* flags value of gtags_open() */
 	int flags;			/* flags */
-	char root[MAXPATHLEN+1];	/* root directory of source tree */
+	char root[MAXPATHLEN];	/* root directory of source tree */
 	/*
 	 * Stuff for GTOP_PATH.
 	 */
@@ -95,30 +96,20 @@ typedef struct {
 	GTP gtp;
 	POOL *segment_pool;
 	VARRAY *vb;
-	char cur_tagname[IDENTLEN+1];	/* current tag name */
+	char cur_tagname[IDENTLEN];	/* current tag name */
 	/*
 	 * Stuff for compact format
 	 */
-	char cur_path[MAXPATHLEN+1];	/* current path */
+	char cur_path[MAXPATHLEN];	/* current path */
 	STRBUF *sb;			/* string buffer */
 	/* used for compact format and path name only read */
 	STRHASH *path_hash;
-	/*
-	 * Stuff for 1-pass parsing
-	 *
-	 * If you set file pointer to tmpfile_fp, gtags_put() and gtags_put_using()
-	 * write records to the temporary file instead of db(3) file.
-	 */
-	FILE *tmpfile_fp;
 } GTOP;
 
 const char *dbname(int);
 GTOP *gtags_open(const char *, const char *, int, int, int);
-void gtags_put(GTOP *, const char *, const char *);
 void gtags_put_using(GTOP *, const char *, int, const char *, const char *);
 void gtags_flush(GTOP *, const char *);
-void gtags_add_ref_sym(GTOP *const *, FILE *);
-void gtags_move_ref_sym(GTOP *const *);
 void gtags_delete(GTOP *, IDSET *);
 GTP *gtags_first(GTOP *, const char *, int);
 GTP *gtags_next(GTOP *);
