@@ -1,21 +1,20 @@
 /*
- * Copyright (c) 2005, 2006 Tama Communications Corporation
+ * Copyright (c) 2005, 2006, 2010 Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
- * GNU GLOBAL is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * GNU GLOBAL is distributed in the hope that it will be useful,
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 #ifndef _PATHCONVERT_H_
 #define _PATHCONVERT_H_
@@ -29,14 +28,17 @@ typedef struct {
 	int type;		/* PATH_ABSOLUTE, PATH_RELATIVE */
 	int format;		/* defined in format.h */
 	STRBUF *abspath;
-	char basedir[MAXPATHLEN+1];
+	char basedir[MAXPATHLEN];
 	int start_point;
 
 } CONVERT;
 
+void set_encode_chars(const unsigned char *);
+char *decode_path(const unsigned char *);
 CONVERT *convert_open(int, int, const char *, const char *, const char *, FILE *);
-void convert_put(CONVERT *cv, const char *line);
-void convert_put_using(CONVERT *, const char *, const char *, int, const char *);
+void convert_put(CONVERT *, const char *);
+void convert_put_path(CONVERT *, const char *);
+void convert_put_using(CONVERT *, const char *, const char *, int, const char *, const char *);
 void convert_close(CONVERT *cv);
 
 #endif /* ! _PATHCONVERT_H_ */
